#include "stm32f4xx.h"
#include "ssd1963.h"
#include "stm32f4xx_gpio.h"



extern volatile uint32_t ms;



uint8_t Lo(uint16_t X)
{
	uint8_t temp;
	temp=(uint8_t)(X&0x00ff) ;
	 return temp;  //mlodsza czesc bajtu X
}
uint8_t Hi(uint16_t X)
{
	uint8_t temp;
	temp=(uint8_t)((X>>8)&0x00ff); //starsza czesc bajtu X
			return temp;
}

void LCD_SetArea(u16 start_x, u16 start_y, u16 end_x, u16 end_y)
{

    LCD_WriteCommand(CMD_SET_COLUMN);
    Clr_Cs;
    LCD_WriteData(start_x>>8);
    LCD_WriteData(start_x);
    LCD_WriteData(end_x>>8);
    LCD_WriteData(end_x);
    Set_Cs;
    LCD_WriteCommand(CMD_SET_PAGE);
    Clr_Cs;
    LCD_WriteData(start_y>>8);
    LCD_WriteData(start_y);
    LCD_WriteData(end_y>>8);
    LCD_WriteData(end_y);
    Set_Cs;
}

void LCD_EnterSleepMode (void)
{
    LCD_WriteCommand(CMD_ENT_SLEEP);
}


void LCD_ExitSleepMode (void)
{
    LCD_WriteCommand(CMD_EXIT_SLEEP);
}



void LCD_WriteCommand(uint8_t cmd)
{
	Clr_Rs;
	GPIOC->ODR=cmd;
	Clr_Cs;
	Clr_nWr;
	Set_nWr;
	Set_Cs;
}
void LCD_WriteCommandCont(uint8_t cmd)
{
	GPIOC->ODR=cmd;
	Clr_Cs; Clr_nWr;
	Set_nWr;
	Set_Cs;
}
void LCD_WriteData(uint16_t data)
{
	Set_Rs;
	GPIOC->ODR=data;
	Clr_nWr;
	Set_nWr;
}
void LCD_WriteDataCont(uint16_t data)
{
	GPIOC->ODR=data;
	Clr_nWr;
	Set_nWr;
}




void LCD_GPIOInit(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    //Jezeli nie inicjalizujemy sygnalu taktujacego dla szyn w funkcji RCC to nalezy to zrobic tutaj
    // RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC | RCC_AHB1Periph_GPIOB, ENABLE);


    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;          //Port uzywany jako wyjsciowy
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;         //Port uzywany jako zwykly port typu Pull Push
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;     //Czestotliwosc taktowania ustawiona na 100Mhz
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;       //Pull up pull down wstepnie jako no pull


    // Inicjalizacja portow danych oraz portow sterujacych
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 ;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    //Inicjalizacja pinu reset
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
    // Inicjalizacja portow danych oraz portow sterujacych
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

     //Predkosc 2Mhz poniewaz PC13, PC14, PC15 sa portami OSC32 i nie moga pracowac szybciej jako IO. Predkosc 2Mhz nalezy wpisac w przypadku
    //problemow z inicjalizacja wyswietlacza
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;       //Pull up pull down wstepnie jako no pull
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;     //Czestotliwosc taktowania ustawiona npowinna byc na 2 mhz

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 |
                                 	GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 |
	                                GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
    GPIO_Init(GPIOC, &GPIO_InitStructure);

} 


void LCD_ResetDevice(void)
{


    Set_Rst;
    Set_Cs;         // Wybranie ssd1963
    Set_nRd;
    Set_nWr;

	 ms=1000;  // delay 100ms
	 while(ms)
	 {
	 }


    Clr_Rst;
	
	//**********
	//**********
	
	 ms=1000;  // delay 100ms
	 while(ms)
	 {
	 }
 
	 //**********
	 //**********
     Set_Rst;       // przejscie ze stanu po zresetowniu do stanu uspiecnia

     ms=50;         //odczekanie 5ms
	 while(ms)
	 {
	 }


    //Ustawienie mnoznikow M i N w petli PLL, VCO = crystal freq * (M+1)
    //PLL freq = VCO/N granic VCO wynosi 250MHz < VCO < 800MHz
    //Maksymalna czestotliwosc petli PLL wynosi 120MHz. Optymalnie jest
    // dobrze ustawic	 100MHz jako czestotliwosc petli PLL



    LCD_WriteCommand(CMD_SET_PLL_MN);   // Ustaw petle PLL z oscylatorem (kwarcem)= 10MHz (hardware)
                                        // Multiplie M=29 r N = 2, VCO (>250MHz)= OSC*(N+1), VCO = 360MHz

    Clr_Cs;
    LCD_WriteData(0x1d);

     LCD_WriteData(0x02);                // Divider N = 2, PLL = 360/(N+1) = 100MHz
    LCD_WriteData(0x54);                // Uruchomienie walidacji M i N wartosci
    Set_Cs;								//0x54 Dumny byte
    LCD_WriteCommand(CMD_PLL_START);    // Komenda dla ustawienie sygnalu PLL
    Clr_Cs;
    LCD_WriteData(0x01);                // enable PLL - zalaczenie PLL a nie innego sygnalu
    Set_Cs;

    ms=2;
    while(ms){}

    LCD_WriteCommand(CMD_PLL_START);    // Start PLL
    Clr_Cs;
    LCD_WriteData(0x03);                // ustaiwnie wyjscia petli PLL jako sygnalu zegarowego dla systemu sterownika
    Set_Cs;

   LCD_WriteCommand(CMD_SOFT_RESET);   // Reset programowy
	//odczekanie 100ms
   ms=1000;
	while(ms){
	}
		
		
		//Ustawienie zegarow
    //Set LSHIFT freq, i.e. the DCLK with PLL freq 120MHz set previously
    //Typical DCLK for TY700TFT800480 is 33.3MHz(datasheet), experiment shows 30MHz gives a stable result
    //30MHz = 120MHz*(LCDC_FPR+1)/2^20
    //LCDC_FPR = 262143 (0x3FFFF)
    //Time per line = (DISP_HOR_RESOLUTION+DISP_HOR_PULSE_WIDTH+DISP_HOR_BACK_PORCH+DISP_HOR_FRONT_PORCH)/30 us = 1056/30 = 35.2us
    LCD_WriteCommand(CMD_SET_PCLK);
    Clr_Cs;

    LCD_WriteData(0x03);
    LCD_WriteData(0xff);
    LCD_WriteData(0xff);
    Set_Cs;
    LCD_WriteCommand(CMD_SET_PANEL_MODE);
    Clr_Cs;
   // LCD_WriteData(0x30);    //ustawienie gdy uzywany wyswietlacz ma interfejs RGB888
	LCD_WriteData(0x10);     //ustawiamy gdy uzywany wyswietlacz ze sterowownikiem ma RGB666
		
    LCD_WriteData(0x80);                // Ustawienie opcji TTL
    LCD_WriteData((DISP_HOR_RESOLUTION-1)>>8); //Ustaiwnie wielkosci panelu
    LCD_WriteData(DISP_HOR_RESOLUTION-1);
    LCD_WriteData((DISP_VER_RESOLUTION-1)>>8);
    LCD_WriteData(DISP_VER_RESOLUTION-1);
    LCD_WriteData(0x00);                //RGB sekwencja
    Set_Cs;

    //Ustaiwenie okresu dla przejscia horyzontalnego
    LCD_WriteCommand(CMD_SET_HOR_PERIOD);
    #define HT (DISP_HOR_RESOLUTION+DISP_HOR_PULSE_WIDTH+DISP_HOR_BACK_PORCH+DISP_HOR_FRONT_PORCH)
    Clr_Cs;
    LCD_WriteData((HT-1)>>8);
    LCD_WriteData(HT-1);
    #define HPS (DISP_HOR_PULSE_WIDTH+DISP_HOR_BACK_PORCH)
    LCD_WriteData((HPS-1)>>8);
    LCD_WriteData(HPS-1);
    LCD_WriteData(DISP_HOR_PULSE_WIDTH-1);
    LCD_WriteData(0x00);
    LCD_WriteData(0x00);
    LCD_WriteData(0x00);
    Set_Cs;
    //Ustaiwneie okresu dla przejscia wertykalnego
    LCD_WriteCommand(CMD_SET_VER_PERIOD);
    #define VT (DISP_VER_PULSE_WIDTH+DISP_VER_BACK_PORCH+DISP_VER_FRONT_PORCH+DISP_VER_RESOLUTION)
    Clr_Cs;
    LCD_WriteData((VT-1)>>8);
    LCD_WriteData(VT-1);
    #define VSP (DISP_VER_PULSE_WIDTH+DISP_VER_BACK_PORCH)
    LCD_WriteData((VSP-1)>>8);
    LCD_WriteData(VSP-1);
    LCD_WriteData(DISP_VER_PULSE_WIDTH-1);
    LCD_WriteData(0x00);
    LCD_WriteData(0x00);
    Set_Cs;

    // Format piksela
    LCD_WriteCommand(CMD_SET_PIXEL_FORMAT);
    Clr_Cs;
    LCD_WriteData(0x50);                //  ustawienie ile bitow przypada na jeden pixel. Ustawiamy 16
    Set_Cs;
		ms=1;
		while(ms);

    //Ustawienie kodowania RGB 565 lub innego
    LCD_WriteCommand(CMD_SET_DATA_INTERFACE);
    Clr_Cs;
	//LCD_WriteData(0x03);
    LCD_WriteData(0x03);                //16-bit(565 format) - nalezy ustawic format jaki bedzie uzywany wedlug tabelki dokumentacji SSD1963 (tabelka kolorow)
    //LCD_WriteData(0x00);              //8-bit 
    Set_Cs;
		
	ms=1;
	while(ms);

    //Ustawienie podawania adresu
    LCD_WriteCommand(CMD_SET_ADDR_MODE);
    Clr_Cs;
   // LCD_WriteData(0x0E);
	 LCD_WriteData(0x06);   //Ustawiamy wyswietlanie RGB z bufora ramki a nie BGR
    Set_Cs;
    LCD_WriteCommand(CMD_ON_DISPLAY);   // Zalaczenie wyswietlacza

}


void LCD_PutPixel(u16 x, u16 y, u16 color)
{
    LCD_SetArea(x,y,x,y);
    LCD_WriteCommand(CMD_WR_MEMSTART);
    Clr_Cs;
    LCD_WriteData(color);
    Set_Cs;
}


void LCD_Clear(u16 color)
{
    u32 xcounter, ycounter;

    LCD_SetArea(0,0,GetMaxX(),GetMaxY());

    LCD_WriteCommand(CMD_WR_MEMSTART);
    Clr_Cs;
    LCD_SetData;
    for(ycounter=0;ycounter<DISP_VER_RESOLUTION;ycounter++)
    {
        for(xcounter=0;xcounter<DISP_HOR_RESOLUTION;xcounter++)
        {
            LCD_WriteDataCont(color);
        }
    }
    Set_Cs;
}



//rysowanie linii
void tft_line(int X1, int Y1,int X2,int Y2,int Color) {
	int CurrentX, CurrentY, Xinc, Yinc,
		Dx, Dy, TwoDx, TwoDy,
		TwoDxAccumulatedError, TwoDyAccumulatedError;

	Dx = (X2-X1);
	Dy = (Y2-Y1);

	TwoDx = Dx + Dx;
	TwoDy = Dy + Dy;

	CurrentX = X1;
	CurrentY = Y1;

	Xinc = 1;
	Yinc = 1;

	if(Dx < 0) {

		Xinc = -1;
		Dx = -Dx;
		TwoDx = -TwoDx;
	}

	if (Dy < 0) {
		Yinc = -1;
		Dy = -Dy;
		TwoDy = -TwoDy;
	}

  LCD_PutPixel( X1, Y1, Color);
	

	if ((Dx != 0) || (Dy != 0)) {

		if (Dy <= Dx) {
			TwoDxAccumulatedError = 0;
			do {
			    CurrentX += Xinc;
			    TwoDxAccumulatedError += TwoDy;
			    if(TwoDxAccumulatedError > Dx) {
			        CurrentY += Yinc;
			        TwoDxAccumulatedError -= TwoDx;
			    }
					LCD_PutPixel( CurrentX, CurrentY, Color);
			
			  } while (CurrentX != X2);
		  } else {
			  TwoDyAccumulatedError = 0;
			  do {
			      CurrentY += Yinc;
			      TwoDyAccumulatedError += TwoDx;
			      if(TwoDyAccumulatedError>Dy) {
				      CurrentX += Xinc;
				      TwoDyAccumulatedError -= TwoDy;
			      }
						LCD_PutPixel( CurrentX, CurrentY, Color);
			    
			  } while (CurrentY != Y2);
		  }
	}
}

//rysowanie pelnego prostokatu
void full_rect( int X1, int Y1, int X2, int Y2, uint16_t color)
{
	int i, j;
		for(i=X1; i<=X2; i++)
		{
				for(j=Y1; j<=Y2; j++)
				{
					LCD_PutPixel( i, j, color);
				}
		}
}	


//Funkcja rysujaca pojedynczy zaokraglony kwadrat
void tft_round_rect1(int x, int y, int width, int height, uint8_t radius, uint8_t fill, uint16_t color)
{
	  	int16_t tSwitch;

	  	int x1 = 0, y1 = radius;
	  	tSwitch = 3 - 2 * radius;

		while (x1 <= y1) {

			LCD_PutPixel( x+radius - x1, y+radius - y1, color);
			LCD_PutPixel(x+radius - y1, y+radius - x1, color);
			LCD_PutPixel(x+width-radius + x1, y+radius - y1, color);
			LCD_PutPixel(x+width-radius + y1,  y+radius - x1, color);
			LCD_PutPixel(x+width-radius + x1,  y+height-radius + y1, color);
			LCD_PutPixel(x+width-radius + y1,  y+height-radius + x1, color);
			LCD_PutPixel(x+radius - x1,  y+height-radius + y1, color);
			LCD_PutPixel(x+radius - y1,  y+height-radius + x1, color);

				tft_line(x+radius - x1, y+radius - y1, x+width-radius + x1, y+radius - y1, color);
				tft_line(x+radius - y1, y+radius - x1, x+width-radius + y1, y+radius - x1 ,color);
				tft_line(x+width-radius + x1, y+height-radius + y1, x+radius - x1, y+height-radius + y1, color);
				tft_line(x+width-radius + y1, y+height-radius + x1, x+radius - y1, y+height-radius + x1, color);


		    if (tSwitch < 0) {
		    	tSwitch += (4 * x1 + 6);
		    } else {
		    	tSwitch += (4 * (x1 - y1) + 10);
		    	y1--;
		    }
		    x1++;
		}

			uint16_t i;
			for(i=0; i<height-(2*radius); i++) tft_line(x, y+radius+i, x+width, y+radius+i, color);

		tft_line(x+radius, y, x+width-radius, y, color);
		tft_line(x+radius, y+height, x+width-radius, y+height, color);

		tft_line(x, y+radius, x, y+height-radius, color);
		tft_line(x+width, y+radius, x+width, y+height-radius, color);
}

//Kwadrat z zaokraglonynmi krawedziami
void tft_round_rect(int x, int y, int width, int height, uint8_t radius, uint16_t color1, uint16_t color2)
{
			tft_round_rect1(x,y,width,height, radius,1, color1);
			tft_round_rect1(x+2,y+2,width-4,height-4, radius,1, color2);
}

//Funkcja rysujacy pelny prostokat
void full_rect3( int X1, int Y1, int X2, int Y2, uint16_t color)
{
	uint16_t temp1, temp2;
	uint32_t tempmul;
	uint32_t k;

	temp1=X2-X1;
	temp2=Y2-Y1;

	tempmul=(temp1+1)*(temp2+1);


	  LCD_SetArea(X1,Y1,X2,Y2);
	  LCD_WriteCommand(CMD_WR_MEMSTART);

	for(k=0; k<=tempmul; k++)
	{
	    Clr_Cs;
	    LCD_WriteData(color);
	    Set_Cs;
	}


}

//Funkcja rysujaca narysowana grafike w monohromatyczna w pikselach
//alfa oznacza czy nalezy wypelnic nieuzyte tlo kolore lub tez nie
uint8_t putmonograph(uint16_t x, uint16_t y, uint16_t color, uint16_t bkcolor, uint16_t width, uint16_t height,const uint8_t *s, uint8_t alpha)
{
	uint8_t tempi, tempk, wier, dtemp=0, fault=0;
	unsigned char wartosc;
	const uint8_t *p;

	//jezeli wysokosc i szerokosc nie jest podzielna przez 8 to zwroc blad
	if(height%8) fault= 1;
	if(width%8) fault= 1;

	wier=width/8;

	for(tempk=0; tempk<height; tempk++)
	{
	for(tempi=0; tempi<wier; tempi++)
	{
		p=s+(tempk*wier)+tempi;
		wartosc=*p;
		for(dtemp=0; dtemp<8; dtemp++)
		{

			if(wartosc & (1<<(7-dtemp)))  LCD_PutPixel(x+(tempi*8)+dtemp, y+tempk, color);
		    else if(!alpha)  LCD_PutPixel(x+(tempi*8)+dtemp, y+tempk, bkcolor);
		}

	}
	}

	return fault;
}

//rysowanie grubej lini
void tft_line2(int X1, int Y1,int X2,int Y2,int Color) {
	int CurrentX, CurrentY, Xinc, Yinc,
		Dx, Dy, TwoDx, TwoDy,
		TwoDxAccumulatedError, TwoDyAccumulatedError;
	uint8_t kk;

	Dx = (X2-X1);
	Dy = (Y2-Y1);

	TwoDx = Dx + Dx;
	TwoDy = Dy + Dy;

	CurrentX = X1;
	CurrentY = Y1;

	Xinc = 1;
	Yinc = 1;

	if(Dx < 0) {

		Xinc = -1;
		Dx = -Dx;
		TwoDx = -TwoDx;
	}

	if (Dy < 0) {
		Yinc = -1;
		Dy = -Dy;
		TwoDy = -TwoDy;
	}


	LCD_SetArea(X1-1,Y1-1,X2+1,Y2+1);
		  LCD_WriteCommand(CMD_WR_MEMSTART);

		for(kk=0; kk<=9; kk++)
		{
		    Clr_Cs;
		    LCD_WriteData(Color);
		    Set_Cs;
		}




	if ((Dx != 0) || (Dy != 0)) {

		if (Dy <= Dx) {
			TwoDxAccumulatedError = 0;
			do {
			    CurrentX += Xinc;
			    TwoDxAccumulatedError += TwoDy;
			    if(TwoDxAccumulatedError > Dx) {
			        CurrentY += Yinc;
			        TwoDxAccumulatedError -= TwoDx;
			    }


					LCD_SetArea(CurrentX-1,CurrentY-1,CurrentX+1,CurrentY+1);
				    LCD_WriteCommand(CMD_WR_MEMSTART);

							for(kk=0; kk<=9; kk++)
							{
							    Clr_Cs;
							    LCD_WriteData(Color);
							    Set_Cs;
							}

			  } while (CurrentX != X2);
		  } else {
			  TwoDyAccumulatedError = 0;
			  do {
			      CurrentY += Yinc;
			      TwoDyAccumulatedError += TwoDx;
			      if(TwoDyAccumulatedError>Dy) {
				      CurrentX += Xinc;
				      TwoDyAccumulatedError -= TwoDy;
			      }
			                         LCD_SetArea(CurrentX-1,CurrentY-1,CurrentX+1,CurrentY+1);
			      				    LCD_WriteCommand(CMD_WR_MEMSTART);

			      							for(kk=0; kk<=9; kk++)
			      							{
			      							    Clr_Cs;
			      							    LCD_WriteData(Color);
			      							    Set_Cs;
			      							}

			  } while (CurrentY != Y2);
		  }
	}
}

//------------------------------------------------------------------------------------------------
//wstawianie obrazka ze skala oraz przezroczystoscia lub bez
void putpictures(u16 x, u16 y, u16 width, u16 height, unsigned char const *grafika, uint8_t scale, uint8_t alpha)   // scale oznacza skale obrazka np 2:1, 3:1, 1:1
{
		u16 i, j, t;
		unsigned char const *s;

		s=grafika;


		for(j=0; j<height; j++)
		{
				for(i=0; i<width; i++)
				{
					t=(*s);
					s++;
					t=t | (*s<<8);
					s++;

					switch(scale)
					{
					case 1:
					{
						LCD_PutPixel( x+i, y+j, t);
					}
					case 2:
					{
						LCD_PutPixel(  x+(2*i), y+(2*j), t);
						if(!alpha) LCD_PutPixel( x+(2*i)+1, y+(2*j), t);
						if(!alpha) LCD_PutPixel(  x+(2*i),y+(2*j)+1, t);
						LCD_PutPixel(  x+(2*i)+1, y+(2*j)+1, t);
					}
					}
				}
		}
}

//funkcja odczekujaca interwal czasu
void nop(uint32_t nop)
{
	uint32_t ki=0;

	for(ki=0; ki<nop; nop++);

}

//odczytanie okreslonego obszaru z wyswietlacza i zapisanie go do tablicy buffor
void LCD_ReadData(uint16_t *buffor, uint32_t *k, uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
uint16_t *s;

GPIO_InitTypeDef GPIO_InitStructure;
uint32_t i=0;
uint32_t maksbuf, deltax, deltay;

s=buffor;
deltax=x2-x1+1;
deltay=y2-y1+1;
maksbuf=deltax*deltay;

*k=deltax;
k++;
*k=deltay;
k++;
*k=maksbuf;

LCD_SetArea(x1, y1, x2, y2);

Clr_Cs;
LCD_WriteCommand(CMD_RD_MEMSTART);
Set_Cs;
Set_Rs;

//Przelaczenie portow na odbieranie na wejsciowe
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;          //Port uzywany jako wyjsciowy
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;         //Port uzywany jako zwykly port typu Pull Push
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;      //Czestotliwosc taktowania
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;         //Pull up pull down wstepnie jako no pull
  GPIO_InitStructure.GPIO_Pin = (uint16_t)0xFFFF;
  GPIO_Init(GPIOC, &GPIO_InitStructure);



	for(i=0; i<maksbuf; i++)
	{
	         	Clr_Cs;
		         Clr_nRd;

		         	asm("NOP");
		         	asm("NOP");

		         *s=GPIOC->IDR;
		          s++;
       	        Set_nRd;
       	        Set_Cs;


	}


//Ponowne ustawienie portow jako wejscia
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;          //Port uzywany jako wyjsciowy
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;         //Port uzywany jako zwykly port typu Pull Push
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;      //Czestotliwosc taktowania
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;         //Pull up pull down wstepnie jako no pull
  GPIO_InitStructure.GPIO_Pin = (uint16_t)0xFFFF;
  GPIO_Init(GPIOC, &GPIO_InitStructure);
}

//narysowanie danego obrazka ktory zostal odczytany z wyswietlacza i zapisany w talibcy ts
void putfrombufor(uint16_t *ts, uint32_t *k, uint16_t x, uint16_t y)
{
uint16_t *s;
s=ts;

uint32_t i=0;
uint32_t temp1, temp2, temp3;
temp1=*k;   //szerokosc
temp1=temp1-1;
k++;
temp2=*k;   //wysokosc
temp2=temp2-1;
k++;
temp3=*k;   //obliczona wartosc elementow bufora

LCD_SetArea(x, y, x+temp1, y+temp2);
LCD_WriteCommand(CMD_WR_MEMSTART);

Set_Rs;

	for(i=0; i<temp3; i++)
	{
		Clr_Cs;
		LCD_WriteData(*s);
		s++;
		Set_Cs;
	}
}


//Szybkie wstawianie grafiki na wyswietlacz
void putfastpicture(u16 x, u16 y, u16 width, u16 height, unsigned char const *grafika)   // scale oznacza skale obrazka np 2:1, 3:1, 1:1
{
		u16 i, j, t;
		unsigned char const *s;

		s=grafika;

LCD_SetArea(x, y, x+width-1, y+height-1);
LCD_WriteCommand(CMD_WR_MEMSTART);

		for(j=0; j<height; j++)
		{
				for(i=0; i<width; i++)
				{
					t=(*s);
					s++;
					t=t | (*s<<8);
					s++;

					Clr_Cs;
		            LCD_WriteData(t);
		            Set_Cs;

				}
		}
}

//Funkcja rysujaca kwadrat przezroczystosci na danym obszarze X1-x2  ; y2-y1, o kolore color
void full_rectalpha( int X1, int Y1, int X2, int Y2, uint16_t color)
{
	uint16_t temp1, temp2;

	uint32_t k,d;

	temp1=X2-X1;
	temp2=Y2-Y1;
	temp1=temp1+1;
	temp2=temp2+1;



	  for(d=0; d<temp2; d++)
	{
		  for(k=0; k<temp1; k++)
		  {

			  if(d%2)
			  {
				  if(k%2)
				  {
					 LCD_PutPixel(X1+k, Y1+d, White);
				  }
			  }
			  else
			  {
				  if(!(k%2))
				  {
					  LCD_PutPixel(X1+k, Y1+d, White);
				  }
			  }

			  Set_Cs;
		  }
	}
}

//Funkcja rysujaca grafike z tablicy zawierajacej tlo ekranu o rozdzielczosci 800x480px
void putrectanglefromtable(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, unsigned char const *tab)
{
	uint32_t width=0, height=0;
	uint16_t i,j, localx, localy, t=0;
	uint32_t tabnumber=0;
	unsigned char const *s;
	s=tab;

	width=x2-x1+1;
	height=y2-y1+1;

	LCD_SetArea(x1, y1, x1+width-1, y1+height-1);
	LCD_WriteCommand(CMD_WR_MEMSTART);

	for(i=0; i<height; i++)
	{
		for(j=0; j<width; j++)
		{
			localx=x1+j;
			localy=y1+i;

			tabnumber=localy*800*2+localx*2;
			s=tab+tabnumber;

			t=(*s);
			s++;
			t=t | (*s<<8);
			s++;

			Clr_Cs;
			LCD_WriteData(t);
			Set_Cs;

		}
	}
}
